@@ -41,6 +41,7 @@
 
 #include <libgnomeprintui/gnome-print-i18n.h>
 #include <libgnomeprintui/gnome-print-job-preview.h>
+#include <libgnomeprintui/gnome-print-ui-private.h>
 
 #define GPP_COLOR_RGBA(color, ALPHA) \
 	((guint32) (ALPHA              | \
@@ -2431,7 +2432,7 @@ gnome_print_job_preview_init (GnomePrintJobPreview *jp)
 {
 	GtkWidget *vbox, *tb, *l, *status;
 	const gchar *env_theme_variable;
-	gchar *text;
+	gchar *text, *ui_file;
 	AtkObject *atko;
 	GList *action_groups;
 	GError *e = NULL;
@@ -2548,8 +2549,9 @@ gnome_print_job_preview_init (GnomePrintJobPreview *jp)
 		GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 	/* Toolbar */
-	gtk_ui_manager_add_ui_from_file (jp->ui_manager,
-			GNOME_PRINT_JOB_PREVIEW_DATADIR "/gnome-print-job-preview.xml", &e);
+	ui_file = g_build_filename (gnome_printui_job_preview_data_dir, "gnome-print-job-preview.xml", NULL);
+	gtk_ui_manager_add_ui_from_file (jp->ui_manager, ui_file, &e);
+	g_free (ui_file);
 	if (!e)
 		tb = gtk_ui_manager_get_widget (jp->ui_manager, "/StandardToolbar");
 	if (e || !tb) {
