@@ -41,7 +41,7 @@
 static void gpa_printer_selector_class_init (GPAPrinterSelectorClass *klass);
 static void gpa_printer_selector_init (GPAPrinterSelector *selector);
 static void gpa_printer_selector_finalize (GObject *object);
-
+static void gpa_printer_selector_map (GtkWidget *widget);
 static gboolean gpa_printer_selector_construct (GPAWidget *gpa);
 static void gpa_printer_selector_sync_printer (GtkListStore *model, 
 					       GtkTreeIter  *iter, 
@@ -111,13 +111,16 @@ gpa_printer_selector_class_init (GPAPrinterSelectorClass *klass)
 {
 	GObjectClass *object_class;
 	GPAWidgetClass *gpa_class;
+	GtkWidgetClass *widget_class;
 
 	object_class = (GObjectClass *) klass;
 	gpa_class = (GPAWidgetClass *) klass;
+	widget_class = (GtkWidgetClass *) klass;
 
 	parent_class = gtk_type_class (GPA_TYPE_WIDGET);
 	gpa_class->construct = gpa_printer_selector_construct;
 	object_class->finalize = gpa_printer_selector_finalize;
+	widget_class->map = gpa_printer_selector_map;
 }
 
 static void
@@ -196,6 +199,7 @@ gpa_printer_selector_init (GPAPrinterSelector *ps)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (ps->treeview), col);
 	
 	gtk_widget_show (ps->treeview);
+	gtk_widget_grab_focus (ps->treeview);
 }
 
 static void
@@ -281,7 +285,7 @@ interactive_search_func (GtkTreeModel *model,
 			 gpointer search_data)
 {
 	char *name;
-	GPANode *printer;
+	GPANode *printer, *location;
 	gboolean ret;
 
 	gtk_tree_model_get (model, iter, 0, &printer, -1);	
@@ -290,6 +294,16 @@ interactive_search_func (GtkTreeModel *model,
 	ret = strncmp (key, name, strlen (key));
 	g_free (name);
 
+	if (ret != 0) {
+		location = gpa_printer_get_state_by_id (GPA_PRINTER (printer), "Location");
+		if (location && printer_has_queue (GPA_PRINTER (printer))) {
+			name = gpa_node_get_value (location);
+			
+			ret = strncmp (key, name, strlen (key));
+			g_free (name);
+		}
+	}
+
 	return ret;
 }
 
@@ -359,9 +373,11 @@ get_printer_icon (GPANode *printer)
 	settings_list = gpa_node_get_child_from_path (printer, "Settings");
 	settings = gpa_list_get_default (GPA_LIST (settings_list));
 	icon_name = gpa_node_get_path_value (settings, "Icon.Filename");
-	if (icon_name != NULL)
+	if (icon_name != NULL) {
 		res = gtk_icon_theme_load_icon (theme, icon_name,
 						18, 0, NULL);
+		g_free (icon_name);
+	}
 	if (res == NULL)
 		res = gtk_icon_theme_load_icon (theme, "gnome-dev-printer",
 						18, 0, NULL);
@@ -502,15 +518,13 @@ scroll_to_selected (GPAPrinterSelector *ps)
 }
 
 static void
-gpa_printer_selector_printer_added_cb (GPANode *parent, GPANode *child,
-				       GPAPrinterSelector *ps)
+gpa_printer_selector_printer_added (GPANode *parent, GPANode *child,
+				    GPAPrinterSelector *ps)
 {
  	GtkTreeIter iter, sort_iter;
  	GtkTreeSelection *selection;
  	GPANode *default_printer = NULL;
 
-	GDK_THREADS_ENTER ();
-
 	g_return_if_fail (node_to_iter (ps->model, child, &iter) == FALSE);
 
 	gtk_list_store_append (GTK_LIST_STORE (ps->model), &iter);
@@ -532,7 +546,14 @@ gpa_printer_selector_printer_added_cb (GPANode *parent, GPANode *child,
  	}
  
  	scroll_to_selected (ps);
+}
 
+static void
+gpa_printer_selector_printer_added_cb (GPANode *parent, GPANode *child,
+				       GPAPrinterSelector *ps)
+{
+	GDK_THREADS_ENTER ();
+	gpa_printer_selector_printer_added (parent, child, ps);
 	GDK_THREADS_LEAVE ();
 }
 
@@ -569,10 +590,21 @@ gpa_printer_selector_construct (GPAWidget *gpa)
 
 	child = gpa_node_get_child (ps->node, NULL);
 	while (child) {
-  		gpa_printer_selector_printer_added_cb (ps->node, child, ps);
+  		gpa_printer_selector_printer_added (ps->node, child, ps);
 		child = gpa_node_get_child (ps->node, child);
 	}
 
 	return TRUE;
 }
 
+
+static void gpa_printer_selector_map (GtkWidget *widget)
+{
+	GPAPrinterSelector *ps;
+
+	ps = GPA_PRINTER_SELECTOR (widget);
+
+	gtk_widget_grab_focus (ps->treeview);
+       
+	GTK_WIDGET_CLASS (parent_class)->map (widget);
+}
